/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "fntedit.h"


// Remark
//
// This module caused some headache. I stopped layouting with the
// designer, because font-comboboxes showed some problems. Depending
// on fill-strategy (string or lists) and on starting index! it
// crashed predictionable between 10 and 70 entries. Qt-Windows
// does not show this phenomenon. I guess, this is some memory
// allocation problem. Anyhow, the implicite version works without
// oddities on both OS.
//
// Not enough, the redrawing of menu-entries and toolbar-icons after
// font-resizing is somehow messy. The mainwindow has to be readjusted
// for menus, the toolbars to be deleted and recreated.


// Constructor
fntWidget::fntWidget(QWidget *parent) : QWidget(parent) {

	setMinimumSize(250, 30);
	setWindowTitle("Appearance");

	// Create widget content
	txt00 = new QLabel("Font ");
	com01 = new QComboBox;
	txt02 = new QLabel(" Style ");
	com03 = new QComboBox;
	txt04 = new QLabel(" Size ");
	spi05 = new QSpinBox;
	txt06 = new QLabel(" Icons ");
	spi07 = new QSpinBox;

	// Set tooltips
	txt00->setToolTip("Application font");
	txt02->setToolTip("Application fontstyle");
	txt04->setToolTip("Application fontsize");
	txt06->setToolTip("Toolbar iconsize");

	// Make grid-layout
	QGridLayout *grid = new QGridLayout;
	grid->addWidget(txt00, 0, 0);
	grid->addWidget(com01, 0, 1);
	grid->addWidget(txt02, 0, 2);
	grid->addWidget(com03, 0, 3);
	grid->addWidget(txt04, 0, 4);
	grid->addWidget(spi05, 0, 5);
	grid->addWidget(txt06, 0, 6);
	grid->addWidget(spi07, 0, 7);
	setLayout(grid);

	// Get settings and set actual font/iconsize
	readSettings();
	setFnt(actFnt);
	setStyle(actStyle);
	setFntSize(actFntSize);
	setIcoSize(actIcoSize);

	// Create font-database for font-combo
	findFnts();

	// Set right combo/spinboxes
	com01->setCurrentIndex(com01->findText(actFnt));
	spi05->setRange(8, 48);
	spi05->setValue(actFntSize);
	spi07->setRange(10, 48);
	spi07->setValue(actIcoSize);

	// Set right style-combo
	findStyles();

	// Connections
	connect(com01, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(setFnt(const QString &)));
	connect(com01, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(findStyles()));
	connect(com03, SIGNAL(currentIndexChanged(const QString &)), this, SLOT(setStyle(const QString &)));
	connect(spi05, SIGNAL(valueChanged(int)), this, SLOT(setFntSize(const int)));
	connect(spi07, SIGNAL(valueChanged(int)), this, SLOT(setIcoSize(const int)));
}

// Standard destructor
fntWidget::~fntWidget() {}


/* *************************************************** */
/* SET-SLOTS                                           */
/* *************************************************** */
void fntWidget::setFnt(const QString &fntFamily) {
	QFont sysFnt;

	sysFnt.setFamily(fntFamily);
	sysFnt.setPixelSize(actFntSize);
	QApplication::setFont(sysFnt);
	actFnt=fntFamily;

	// Redraw menus because new fontfamily result in wrong width!!!
	emit repaintMain();
}


void fntWidget::setStyle(const QString &fntStyle) {
	QFont sysFnt;
	const int dummy=6;

    sysFnt=fntDatabase.font(QApplication::font().family(), fntStyle, dummy);
	sysFnt.setPixelSize(actFntSize);
	QApplication::setFont(sysFnt);
	actStyle=fntStyle;

	// Redraw menus because new fontstyle may result wrong width!!!
	emit repaintMain();
}


void fntWidget::setFntSize(const int newSize) {
	QFont sysFnt=QApplication::font();

	sysFnt.setFamily(actFnt);
	sysFnt.setPixelSize(newSize);
	QApplication::setFont(sysFnt);
	actFntSize=newSize;

	// Redraw menus because new fontsize result in wrong width!!!
	emit repaintMain();
}


/* *************************************************** */
/* UTILITY-STUFF                                       */
/* *************************************************** */
// Get current systemfont size
int fntWidget::getSysFntSize() {
	QFont sysFnt=QApplication::font();
	QFontInfo fntInfo=QFontInfo(sysFnt);
	return fntInfo.pixelSize();
}

// Get current stystemfont name
QString fntWidget::getSysFnt() {
	QFont sysFnt=QApplication::font();
	QFontInfo fntInfo=QFontInfo(sysFnt);
	return fntInfo.family();
}


/* *************************************************** */
/* FINDING-STUFF                                       */
/* *************************************************** */
// Build fontdatabase and font-combo
void fntWidget::findFnts() {
	QString family;
	com01->clear();

	foreach(family, fntDatabase.families())
		com01->addItem(family);
}

// Build style-combo
void fntWidget::findStyles() {
	QString style;
	com03->clear();

	foreach(style, fntDatabase.styles(com01->currentText()))
		com03->addItem(style);

	int styleIndex=com03->findText(actStyle);

	if(styleIndex==-1)
		com03->setCurrentIndex(0);
	else
		com03->setCurrentIndex(styleIndex);
}


/* *************************************************** */
/* HANDLE SAVE/RESTORE OF WINDOW/FONT-PROPERTIES       */
/* *************************************************** */
void fntWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}

void fntWidget::readSettings() {
	QSettings settings("Heinz", NAME);
	QPoint pos=settings.value("FntWindowPos", QPoint(200, 200)).toPoint();
	QSize size=settings.value("FntWindowSize", QSize(400, 400)).toSize();

	// Ensure to start with defaults on 1st program-start ever
	everStarted=false;
	everStarted=settings.value("FirstFntEdiStart").toBool();
	if(everStarted==true) {
		resize(size);
		move(pos);
		actFnt=settings.value("NewFont").toString();
		actStyle=settings.value("NewStyle").toString();
		actFntSize=settings.value("NewFontSize").toInt();
		actIcoSize=settings.value("NewIconSize").toInt();
	}
	else {
		everStarted=true;
		resize(QSize(525, 70));
		move(QPoint(250, 330));
		actFnt=getSysFnt();
		actStyle="Normal";
		actFntSize=getSysFntSize();
		actIcoSize=22;
	}
}

void fntWidget::writeSettings() {
	QSettings settings("Heinz", NAME);

	// Windows stuff
	settings.setValue("FntWindowPos", pos());
	settings.setValue("FntWindowSize", size());
	settings.setValue("FirstFntEdiStart", everStarted);

	// Font/icon-stuff
	settings.setValue("NewFont", actFnt);
	settings.setValue("NewStyle", actStyle);
	settings.setValue("NewFontSize", actFntSize);
	settings.setValue("NewIconSize", actIcoSize);
}
