/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef FNTEDITOR_H
#define FNTEDITOR_H

#include <QtGui>
#include <QWidget>
#include <QCloseEvent>
#include <QSettings>
#include <QFontDatabase>
#include "property.h"
#include "version.h"
#include "debug.h"


// Handles application appearance
class fntWidget : public QWidget {

Q_OBJECT

public:
	fntWidget(QWidget *parent=0);
	~fntWidget();

	QSize getIcoSize() { return QSize(actIcoSize, actIcoSize); }

protected:
	void closeEvent(QCloseEvent *event);

signals:
	// Inform The MainWindow-Widget to repaint itself
	// Needed, because menu-drawing results in odd width otherwise
	// Like menu-redrawing, toolbars need strange workaround too
	void repaintMain();
	void iconSize(const int newSize);

private slots:
    void setFnt(const QString &fntFamily);
    void setStyle(const QString &fntStyle);
	void setFntSize(const int size);
	void setIcoSize(const int size) { actIcoSize=size; emit iconSize(size); }
	void findStyles();

private:
	QFontDatabase fntDatabase;

	// Current font/toolbar-properties
	QString actFnt;
	QString actStyle;
	int actFntSize;
	int actIcoSize;

	// Utility-stuff
    void findFnts();		// Build font-database for combos
	QString getSysFnt();
	int getSysFntSize();

	// 1.st row of a grid layout; more may follow
	QLabel		*txt00;
	QComboBox	*com01;
	QLabel		*txt02;
	QComboBox	*com03;
	QLabel		*txt04;
	QSpinBox	*spi05;
	QLabel	*txt06;
	QSpinBox *spi07;

	// Settings stuff
	void readSettings();
	void writeSettings();
	bool everStarted;
};

#endif
