/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "funedit.h"


// Constructor
funWidget::funWidget(QWidget *parent) : QWidget(parent) {

	// Set up entire editor
	Ui_funUI::setupUi(this);
	setMinimumSize(200, 90);
	setWindowTitle("Function editor");
	readSettings();		// Get remembered window-settings

	// After <Return> deliver whole string to oglWid
	connect(F0, SIGNAL(returnPressed()), this, SLOT(deliverF0Slot()));
	connect(F1, SIGNAL(returnPressed()), this, SLOT(deliverF1Slot()));
	connect(F2, SIGNAL(returnPressed()), this, SLOT(deliverF2Slot()));

	connect(this, SIGNAL(F0changed()), oglWid, SLOT(setNewF0()));
	connect(this, SIGNAL(F1changed()), oglWid, SLOT(setNewF1()));
	connect(this, SIGNAL(F2changed()), oglWid, SLOT(setNewF2()));
}

// Standard destructor
funWidget::~funWidget() {}


void funWidget::deliverF0Slot() {
	QString tmp = F0->text();

	// Some input checking
	if(tmp.size()>=MAXLEN) {
		QMessageBox::critical(this, tr("F0/U"), tr("String too long"));
		return;
	}

	if(tmp.size()==0) {
		QMessageBox::critical(this, tr("F0/U"), tr("Input # for comment at least!"));
		return;
	}

	// No comment?
	if(!tmp.startsWith("#")) {
		tmp=tmp.toUpper();
	    tmp.remove(QChar(' '));
	}

	// Function mode
	if(!sta.parMod) {
		q2cstrcpy(fun[0].str, tmp);
		if(checkParenthesis(fun[0].str))
			emit F0changed();
		else
			QMessageBox::critical(this, tr("F0"), tr("Parentheses don't match"));
		return;
	}

	// Parameter mode
	q2cstrcpy(par.str[0], tmp);
	if(checkParenthesis(par.str[0]))
		emit F0changed();
	else
		QMessageBox::critical(this, tr("U"), tr("Parentheses don't match"));
}

void funWidget::deliverF1Slot() {
	QString tmp = F1->text();

	// Some input checking
	if(tmp.size()>=MAXLEN) {
		QMessageBox::critical(this, tr("F1/V"), tr("String too long"));
		return;
	}

	if(tmp.size()==0) {
		QMessageBox::critical(this, tr("F1/V"), tr("Input # for comment at least!"));
		return;
	}

	// No comment?
	if(!tmp.startsWith("#")) {
		tmp=tmp.toUpper();
	    tmp.remove(QChar(' '));
	}

	// Function mode
	if(!sta.parMod) {
		q2cstrcpy(fun[1].str, tmp);
		if(checkParenthesis(fun[1].str))
			emit F1changed();
		else
			QMessageBox::critical(this, tr("F1"), tr("Parentheses don't match"));
		return;
	}

	// Parameter mode
	q2cstrcpy(par.str[1], tmp);
	if(checkParenthesis(par.str[1]))
		emit F1changed();
	else
		QMessageBox::critical(this, tr("V"), tr("Parentheses don't match"));
}

void funWidget::deliverF2Slot() {
	QString tmp = F2->text();

	// Some input checking
	if(tmp.size()>=MAXLEN) {
		QMessageBox::critical(this, tr("F2/W"), tr("String too long"));
		return;
	}

	if(tmp.size()==0) {
		QMessageBox::critical(this, tr("F2/W"), tr("Input # for comment at least!"));
		return;
	}

	// No comment?
	if(!tmp.startsWith("#")) {
		tmp=tmp.toUpper();
	    tmp.remove(QChar(' '));
	}

	// Function mode
	if(!sta.parMod) {
		q2cstrcpy(fun[2].str, tmp);
		if(checkParenthesis(fun[2].str))
			emit F2changed();
		else
			QMessageBox::critical(this, tr("F2"), tr("Parentheses don't match"));
		return;
	}

	// Parameter mode
	q2cstrcpy(par.str[2], tmp);
	if(checkParenthesis(par.str[2]))
		emit F2changed();
	else
		QMessageBox::critical(this, tr("W"), tr("Parentheses don't match"));
}


// Check for corresponding parenthesis
bool funWidget::checkParenthesis(const char *string) {
	char *cp=(char *)string;
	int i=0;

	// Comment?
	if(*cp=='#')
		return true;

	// Else check
	do {
		if(*cp=='(') i++;
		if(*cp==')') i--;
		if(i<0) return false;
	} while(*++cp);

	return (i==0);
}


/* *************************************************** */
/* UPDATE ENTRIES IN FUNCTION-EDITOR                   */
/* *************************************************** */
void funWidget::updFunWid() {

	// Functions
	if(!sta.parMod) {
		F0Label->setText(" F0");
		F1Label->setText(" F1");
		F2Label->setText(" F2");
		F0->setText(fun[0].str);
		F1->setText(fun[1].str);
		F2->setText(fun[2].str);
	}
	// Parameter system
	else {
		F0Label->setText(" U");
		F1Label->setText(" V");
		F2Label->setText(" W");
		F0->setText(par.str[0]);
		F1->setText(par.str[1]);
		F2->setText(par.str[2]);
	}
}


/* *************************************************** */
/* HANDLE SAVE/RESTORE OF WINDOW-COORDINATES           */
/* *************************************************** */
void funWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}

void funWidget::readSettings() {
	QSettings settings("Heinz", NAME);
	QPoint pos=settings.value("FunWindowPos", QPoint(200, 200)).toPoint();
	QSize size=settings.value("FunWindowSize", QSize(400, 400)).toSize();

	// Ensure to start with defaults on 1st program-start ever
	everStarted=false;
	everStarted=settings.value("FirstFunEdiStart").toBool();
	if(everStarted==true) {
		resize(size);
		move(pos);
	}
	else {
		everStarted=true;
		resize(QSize(422, 130));
		move(QPoint(10, 520));
	}
}

void funWidget::writeSettings() {
	QSettings settings("Heinz", NAME);
	settings.setValue("FunWindowPos", pos());
	settings.setValue("FunWindowSize", size());
	settings.setValue("FirstFunEdiStart", everStarted);
}
