/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "ligedit.h"


// Constructor
ligWidget::ligWidget(QWidget *parent) : QWidget(parent) {

	// Set up light-editor
	Ui_ligUI::setupUi(this);
	setMinimumSize(230, 450);
	setWindowTitle("Lights");
	readSettings();			// Get remembered settings

	// Set all lights
	connect(set0, SIGNAL(clicked()), oglWid, SLOT(setLig0()));
	connect(set1, SIGNAL(clicked()), oglWid, SLOT(setLig1()));
	connect(set2, SIGNAL(clicked()), oglWid, SLOT(setLig2()));
	connect(set3, SIGNAL(clicked()), oglWid, SLOT(setLig3()));
	connect(set4, SIGNAL(clicked()), oglWid, SLOT(setLig4()));
	connect(set5, SIGNAL(clicked()), oglWid, SLOT(setLig5()));
	connect(set6, SIGNAL(clicked()), oglWid, SLOT(setLig6()));
	connect(set7, SIGNAL(clicked()), oglWid, SLOT(setLig7()));

	// Edit single light
	connect(edit0, SIGNAL(clicked()), oglWid, SLOT(ediLig0()));
	connect(edit1, SIGNAL(clicked()), oglWid, SLOT(ediLig1()));
	connect(edit2, SIGNAL(clicked()), oglWid, SLOT(ediLig2()));
	connect(edit3, SIGNAL(clicked()), oglWid, SLOT(ediLig3()));
	connect(edit4, SIGNAL(clicked()), oglWid, SLOT(ediLig4()));
	connect(edit5, SIGNAL(clicked()), oglWid, SLOT(ediLig5()));
	connect(edit6, SIGNAL(clicked()), oglWid, SLOT(ediLig6()));
	connect(edit7, SIGNAL(clicked()), oglWid, SLOT(ediLig7()));

	// Intensity ambient
	connect(IAR, SIGNAL(valueChanged(int)), oglWid, SLOT(lared(int)));
	connect(IAG, SIGNAL(valueChanged(int)), oglWid, SLOT(lagreen(int)));
	connect(IAB, SIGNAL(valueChanged(int)), oglWid, SLOT(lablue(int)));
	connect(IAA, SIGNAL(valueChanged(int)), oglWid, SLOT(laalpha(int)));

	// Intensity diffuse
	connect(IDR, SIGNAL(valueChanged(int)), oglWid, SLOT(ldred(int)));
	connect(IDG, SIGNAL(valueChanged(int)), oglWid, SLOT(ldgreen(int)));
	connect(IDB, SIGNAL(valueChanged(int)), oglWid, SLOT(ldblue(int)));
	connect(IDA, SIGNAL(valueChanged(int)), oglWid, SLOT(ldalpha(int)));

	// Intensity specular
	connect(ISR, SIGNAL(valueChanged(int)), oglWid, SLOT(lsred(int)));
	connect(ISG, SIGNAL(valueChanged(int)), oglWid, SLOT(lsgreen(int)));
	connect(ISB, SIGNAL(valueChanged(int)), oglWid, SLOT(lsblue(int)));
	connect(ISA, SIGNAL(valueChanged(int)), oglWid, SLOT(lsalpha(int)));

	// Spotlights; A=angle I=intensity
	connect(spotA,   SIGNAL(valueChanged(int)), oglWid, SLOT(setSpotA(int)));
	connect(spotI,   SIGNAL(valueChanged(int)), oglWid, SLOT(setSpotI(int)));
	connect(spotX,   SIGNAL(valueChanged(int)), oglWid, SLOT(setSpotX(int)));
	connect(spotY,   SIGNAL(valueChanged(int)), oglWid, SLOT(setSpotY(int)));
	connect(spotZ,   SIGNAL(valueChanged(int)), oglWid, SLOT(setSpotZ(int)));
	connect(spotBox, SIGNAL(toggled(bool)),     oglWid, SLOT(togSpot(bool)));

	// Attenuation; con=constant lin=linear qua=quadratic
	connect(con, SIGNAL(valueChanged(int)), oglWid, SLOT(setConAtt(int)));
	connect(lin, SIGNAL(valueChanged(int)), oglWid, SLOT(setLinAtt(int)));
	connect(qua, SIGNAL(valueChanged(int)), oglWid, SLOT(setQuaAtt(int)));

	// Light miscellaneous
	connect(distance, SIGNAL(valueChanged(int)), oglWid, SLOT(lightDistance(int)));
	connect(locAmb,   SIGNAL(clicked()), oglWid, SLOT(ligAmbLocked()));
	connect(locDif,   SIGNAL(clicked()), oglWid, SLOT(ligDifLocked()));
	connect(locSpe,   SIGNAL(clicked()), oglWid, SLOT(ligSpeLocked()));
	connect(oglWid,   SIGNAL(updLigWid()), this, SLOT(updLigWid()));

	// To prevent flickering when light sliders are locked
	connect(oglWid,   SIGNAL(updLigAmb()), this, SLOT(updLigAmb()));
	connect(oglWid,   SIGNAL(updLigDif()), this, SLOT(updLigDif()));
	connect(oglWid,   SIGNAL(updLigSpe()), this, SLOT(updLigSpe()));
}

// Standard destructor
ligWidget::~ligWidget() {}


/* *************************************************** */
/* UPDATE SLIDER-VALUES IN LIGHT-EDITOR                */
/* *************************************************** */
void ligWidget::updLigWid() {

	// Check setting-section
	set0->setChecked(false);
	set1->setChecked(false);
	set2->setChecked(false);
	set3->setChecked(false);
	set4->setChecked(false);
	set5->setChecked(false);
	set6->setChecked(false);
	set7->setChecked(false);

	if(lig[0].setLig) set0->setChecked(true);
	if(lig[1].setLig) set1->setChecked(true);
	if(lig[2].setLig) set2->setChecked(true);
	if(lig[3].setLig) set3->setChecked(true);
	if(lig[4].setLig) set4->setChecked(true);
	if(lig[5].setLig) set5->setChecked(true);
	if(lig[6].setLig) set6->setChecked(true);
	if(lig[7].setLig) set7->setChecked(true);

	// Check edit-section
	edit0->setChecked(false);
	edit1->setChecked(false);
	edit2->setChecked(false);
	edit3->setChecked(false);
	edit4->setChecked(false);
	edit5->setChecked(false);
	edit6->setChecked(false);
	edit7->setChecked(false);

	switch(sta.ediLig) {
		case(0): edit0->setChecked(true); break;
		case(1): edit1->setChecked(true); break;
		case(2): edit2->setChecked(true); break;
		case(3): edit3->setChecked(true); break;
		case(4): edit4->setChecked(true); break;
		case(5): edit5->setChecked(true); break;
		case(6): edit6->setChecked(true); break;
		case(7): edit7->setChecked(true); break;
	}

	// RGB-locked and distance
	locAmb->setChecked(lig[sta.ediLig].ambLigLoc);
	locDif->setChecked(lig[sta.ediLig].difLigLoc);
	locSpe->setChecked(lig[sta.ediLig].speLigLoc);
	distance->setValue(lig[sta.ediLig].ligDis);

	// Ambient lights
	IAR->setValue((int)(lig[sta.ediLig].ambLig[0]*255.0f));
	IAG->setValue((int)(lig[sta.ediLig].ambLig[1]*255.0f));
	IAB->setValue((int)(lig[sta.ediLig].ambLig[2]*255.0f));
	IAA->setValue((int)(lig[sta.ediLig].ambLig[3]*255.0f));

	// Diffuse lights
	IDR->setValue((int)(lig[sta.ediLig].difLig[0]*255.0f));
	IDG->setValue((int)(lig[sta.ediLig].difLig[1]*255.0f));
	IDB->setValue((int)(lig[sta.ediLig].difLig[2]*255.0f));
	IDA->setValue((int)(lig[sta.ediLig].difLig[3]*255.0f));

	// Specular lights
	ISR->setValue((int)(lig[sta.ediLig].speLig[0]*255.0f));
	ISG->setValue((int)(lig[sta.ediLig].speLig[1]*255.0f));
	ISB->setValue((int)(lig[sta.ediLig].speLig[2]*255.0f));
	ISA->setValue((int)(lig[sta.ediLig].speLig[3]*255.0f));

	// Spotlight settings
	spotA->setValue((int)(lig[sta.ediLig].spoCut));
	spotI->setValue((int)(lig[sta.ediLig].spoExp/3.0f));
	spotX->setValue((int)(lig[sta.ediLig].spoDir[0]*100.0f));
	spotY->setValue((int)(lig[sta.ediLig].spoDir[1]*100.0f));
	spotZ->setValue((int)(lig[sta.ediLig].spoDir[2]*100.0f));
	spotBox->setChecked(lig[sta.ediLig].spoSta);

	// Attenuations
	con->setValue((int)(lig[sta.ediLig].conAtt*100.0f));
	lin->setValue((int)(lig[sta.ediLig].linAtt*300.0f));
	qua->setValue((int)(lig[sta.ediLig].quaAtt*1000.0f));

	if(ent.modInf==1.0f) {
		spotBox->setEnabled(true);
		attenuationBox->setEnabled(true);
		distantBox->setEnabled(true);
	}
	else {
		spotBox->setEnabled(false);
		attenuationBox->setEnabled(false);
		distantBox->setEnabled(false);
	}
}

// To prevent flickering when light sliders are locked
void ligWidget::updLigAmb() {
	IAR->setValue((int)(lig[sta.ediLig].ambLig[0]*255.0f));
	IAG->setValue((int)(lig[sta.ediLig].ambLig[1]*255.0f));
	IAB->setValue((int)(lig[sta.ediLig].ambLig[2]*255.0f));
	IAA->setValue((int)(lig[sta.ediLig].ambLig[3]*255.0f));
}

void ligWidget::updLigDif() {
	IDR->setValue((int)(lig[sta.ediLig].difLig[0]*255.0f));
	IDG->setValue((int)(lig[sta.ediLig].difLig[1]*255.0f));
	IDB->setValue((int)(lig[sta.ediLig].difLig[2]*255.0f));
	IDA->setValue((int)(lig[sta.ediLig].difLig[3]*255.0f));
}

void ligWidget::updLigSpe() {
	ISR->setValue((int)(lig[sta.ediLig].speLig[0]*255.0f));
	ISG->setValue((int)(lig[sta.ediLig].speLig[1]*255.0f));
	ISB->setValue((int)(lig[sta.ediLig].speLig[2]*255.0f));
	ISA->setValue((int)(lig[sta.ediLig].speLig[3]*255.0f));
}


/* *************************************************** */
/* HANDLE SAVE/RESTORE OF WINDOW-COORDINATES           */
/* *************************************************** */
void ligWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}

void ligWidget::readSettings() {
	QSettings settings("Heinz", NAME);
	QPoint pos=settings.value("LightWindowPos", QPoint(200, 200)).toPoint();
	QSize size=settings.value("LightWindowSize", QSize(400, 400)).toSize();

	// Ensure to start with defaults on 1st program-start ever
	everStarted=false;
	everStarted=settings.value("FirstLigEdiStart").toBool();
	if(everStarted==true) {
		resize(size);
		move(pos);
	}
	else {
		everStarted=true;
		resize(QSize(335, 730));
		move(QPoint(170, 0));
	}
}

void ligWidget::writeSettings() {
	QSettings settings("Heinz", NAME);
	settings.setValue("LightWindowPos", pos());
	settings.setValue("LightWindowSize", size());
	settings.setValue("FirstLigEdiStart", everStarted);
}
