/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/


// Includes
#include <QApplication>
#include <QIcon>
#include <QMessageBox>
#include <QObject>

#include "mainwindow.h"
#include "glwidget.h"
#include "ligedit.h"
#include "matedit.h"
#include "entedit.h"
#include "aniedit.h"
#include "picedit.h"
#include "fntedit.h"
#include "funedit.h"
#include "timers.h"
#include "sysinfo.h"
#include "debug.h"


// Miscellaneous globals
QString appPath;		// Path where Zhu3D was started from
QString argStr;			// Holds arguments from bash or window-manager
bool gotArg=false;		// Where there any arguments?
float MHz=0.0f;			// Stays 0, when no supported hw-timer is found

// Global widgets
MainWindow *mainWin;	// Global because the system-font-stuff needs it
OGLWidget *oglWid;		// OpenGL viewer
ligWidget *ligEdi;		// Light editor
matWidget *matEdi;		// Material editor
entWidget *entEdi;		// Entire editor
aniWidget *aniEdi;		// Animation editor
picWidget *picEdi;		// Pic-dimension editor
fntWidget *fntEdi;		// Fontsize editor
funWidget *funEdi;		// Function/parameter editor
#ifdef DEBUG
errWidget *errEdi;		// Error messages
#endif


/* *************************************************** */
/* MAIN                                                */
/* *************************************************** */
int main(int argc, char *argv[]) {

	// Basic initializations
	QApplication app(argc, argv);

	// Initialize logo and application-path
	QIcon appIcon(":/images/icon.png");
	app.setWindowIcon(appIcon);
	appPath=QCoreApplication::applicationDirPath();

	// Check arguments
	if(argc==2) {
		argStr=argv[1];
		gotArg=true;
	}

	// Create globals
	oglWid = new OGLWidget;		// OpenGL viewer 1st, some inits done there!
	ligEdi = new ligWidget;		// Light editor
	matEdi = new matWidget;		// Material editor
	entEdi = new entWidget;		// Entire editor
	aniEdi = new aniWidget;		// Animation editor
	picEdi = new picWidget;		// Pic-dimension editor
	fntEdi = new fntWidget;		// Fontsize editor
	funEdi = new funWidget;
	#ifdef DEBUG
	errEdi = new errWidget;		// Error messages
	#endif

	// Check OpenGL
	if(!ogl_avail()) {
		QMessageBox::critical(
					0,
					QObject::tr("Program start\n"),
					QObject::tr("<b>Severe error</b>"
								"<p>Zhu3D is missing OpenGL and quits now.</p>")
		);
		return false;
	}

	// Check for hardware acceleration
	if(!oglhw_avail())
		QMessageBox::information(
					0,
					QObject::tr("Program start\n"),
					QObject::tr("<b>OpenGL message:</b>"
						"<p>Missing hardware acceleration. "
						"Animation may be very slow.</p>")
		);

	// Initialize Windows-Multimedia-Timer
	#ifdef WIN32
	initMMTimer();
	#endif

	// Check out clock frequency. Hardware dependent cpuTicks()
	// has to be implemented in timers.cpp and timers.h for this
	#if defined(__i386__) || defined(__x86_64__) || defined(__powerpc__)
	MHz=aniEdi->getClock();
	#endif

	// Create and bring up the mainwindow finally
	mainWin = new MainWindow;
	mainWin->show();

	return app.exec();
}
