/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QWidget>
#include <QString>
#include <QTimer>
#include <iostream>
#include <math.h>

#include "ui_maincontrol.h"
#include "glwidget.h"
#include "ligedit.h"
#include "matedit.h"
#include "entedit.h"
#include "aniedit.h"
#include "picedit.h"
#include "fntedit.h"
#include "funedit.h"
#include "gldraw.h"
#include "sysinfo.h"		// Check out system
#include "version.h"		// Name and version
#include "debug.h"			// Turn on/off

class QAction;
class QMenu;

// Miscellaneous
extern QString appPath;	// Where is Zhu3D started from?
extern QString argStr;		// File to open from command-line
extern bool gotArg;		// Is there any file?

// Widgets
extern OGLWidget *oglWid;	// OpenGL-Viewer
extern ligWidget *ligEdi;	// Light editor
extern matWidget *matEdi;	// Material editor
extern entWidget *entEdi;	// Entire editor
extern aniWidget *aniEdi;	// Animation editor
extern picWidget *picEdi;	// Pic-dimension editor
extern fntWidget *fntEdi;	// Font editor
extern funWidget *funEdi;	// Function editor


/* *************************************************** */
/* MAINWINDOW-CLASS; Ui::name=Objectname from Designer */
/* *************************************************** */
class MainWindow : public QMainWindow, private Ui::mainCtrl {

	Q_OBJECT

public:
	MainWindow();

protected:
	void closeEvent(QCloseEvent *event) { event->accept(); exitSlot(); };

public slots:
	void updMaiWid();			// Update main-widget
	void updMaiInf();			// Update info/cross-section only
	void saveAsSlot();			// Public for animation-editor only

private slots:
	void openSlot();
	void exitSlot();
	void helpSlot();
	void systemSlot();
	void aboutSlot();

	void wireModeSlot();
	void parserMessageSlot();

	// Settings-menu
	void pictureSlot();			// Show pic-dimension editor
	void fineScale();			// Set scaling
	void normalScale();
	void coarseScale();
	void resetViewSlot();		// Handle resets
	void resetScaleSlot();
	void resetAllSlot();
	void fontSlot();			// Show appfont-editor

	// Editor-menus
	void lightSlot();			// Show light editor
	void materialSlot();		// Show material editor
	void animateSlot();			// Show animation editor
	void entireSlot();			// Show entire editor
	void solverSlot();			// Show solver editor
	void functionSlot();		// Show function editor
	void toFrontSlot();			// Bring all minimized editors to front
	void toBackSlot();			// Hide/minimize all open editors
	void showAllSlot();			// Show all editors

	void runStopSlot();			// Toggle animation on/off
	void benchmarkSlot();
	void benchTimerSlot() { benchReady=true; }

	//	Adjust size, when sysFnt is changed in fntEdi
	void repaintSlot() { adjustSize(); }
	void iconSizeSlot(const int newSize);

// Informations for the OpenGL-viewer
signals:
	void gotNewFile();			// Inform viewer about new file loaded
	void gotNewView();			// Inform viewer to update OpenGL only

private:
	void createMenus();
	void createActions();
	void createToolBars();

	// File-stuff
	bool loadFile(const QString &fileName);
	bool saveFile(const QString &fileName);
	void setCurrentFile(const QString &fileName);
	QString strippedName(const QString &fullFileName);
	QString currentFile;

	QWidget   *controlCenter;	// Main controls
	QTextEdit *hlpEdi;			// Help editor
	QTextEdit *solEdi;			// Solver editor

	// Menus
	QMenu *fileMenu;
	QMenu *editorMenu;
	QMenu *settingsMenu;
	QMenu *scaleMenu;			// Submenu from Settings
	QMenu *resetMenu;			// Submenu from Settings
	QMenu *systemMenu;
	QMenu *helpMenu;

	// Actions for menus
	QToolBar *fileToolBar;
	QToolBar *zoomToolBar;
	QToolBar *transToolBar;
	QToolBar *extraToolBar;

	QAction *openAction;
	QAction *saveAsAction;
	QAction *exitAction;

	QAction *lightAction;
	QAction *materialAction;
	QAction *animateAction;
	QAction *entireAction;
	QAction *solEdiAction;
	QAction *funEdiAction;
	QAction *toFrontAction;
	QAction *toBackAction;
	QAction *showAllAction;

	QAction *pictureAction;
	QAction *savePicAction;
	QAction *fineAction;
	QAction *normalAction;
	QAction *coarseAction;
	QAction *fontAction;

	QAction *runStopAction;
	QAction *benchmarkAction;

	QAction *helpAction;
	QAction *systemAction;
	QAction *aboutAction;
	QAction *aboutQtAction;

	// Actions for toolbars
	QAction *zoomInAction;
	QAction *zoomOutAction;

	QAction *transLeftAction;
	QAction *transRightAction;
	QAction *transUpAction;
	QAction *transDownAction;

	QAction *resetViewAction;
	QAction *resetScaleAction;
	QAction *resetAllAction;
	QAction *solveAction;

	// Booleans for settings menu
	bool fineChecked;
	bool normalChecked;
	bool coarseChecked;

	bool benchReady;	// Flag for singleshot-timer

	// Settings stuff
	void readSettings();
	void writeSettings();
	bool everStarted;	// 1.start ever started?
};

#endif
