/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "picedit.h"


// Get actual oglWid-dimension
extern int actwidth;
extern int actheight;


// Constructor
picWidget::picWidget(QWidget *parent) : QWidget(parent) {

	// Set up entire editor
	Ui_picUI::setupUi(this);
	setMinimumSize(110, 170);
	setWindowTitle("Pics");
	everStarted=false;		// Show defaults on 1.st start
	readSettings();			// Get remembered settings

	// Current OpenGL-Window-size
	pic.xDim=actwidth;
	pic.yDim=actheight;

	// Update remembered values
	qual->setValue(pic.quality);
	pngFmt->setChecked(pic.png);
	jpgFmt->setChecked(!pic.png);

	xset=yset=true;			// For avoiding recursion

	// Connections
	connect(xDim, SIGNAL(valueChanged(int)), this, SLOT(xDimSlot(int)));
	connect(yDim, SIGNAL(valueChanged(int)), this, SLOT(yDimSlot(int)));
	connect(qual, SIGNAL(valueChanged(int)), this, SLOT(qualSlot(int)));
	connect(locXY, SIGNAL(clicked()), this, SLOT(picXYLocked()));
	connect(pngFmt, SIGNAL(clicked()), this, SLOT(pngSlot()));
	connect(jpgFmt, SIGNAL(clicked()), this, SLOT(jpgSlot()));
}

// Standard destructor
picWidget::~picWidget() {}


/* *************************************************** */
/* SET DIMENSIONS                                      */
/* *************************************************** */
void picWidget::xDimSlot(int xVal) {
	const float rel=actwidth/(float)actheight;

	pic.xDim=xVal;
	if(pic.locXY && xset) {
		pic.yDim=(int)((float)pic.xDim/rel);
		yset=false;
		yDim->setValue(pic.yDim);
	}
	yset=true;
}

void picWidget::yDimSlot(int yVal) {
	const float rel=actwidth/(float)actheight;

	pic.yDim=yVal;
	if(pic.locXY && yset) {
		pic.xDim=(int)((float)pic.yDim*rel);
		xset=false;
		xDim->setValue(pic.xDim);
	}
	xset=true;
}


/* *************************************************** */
/* UPDATE SLIDER-VALUES IN PIC-DIM-EDITOR              */
/* *************************************************** */
void picWidget::updPicWid() {

	// Update window
	xDim->setValue(pic.xDim=actwidth);
	yDim->setValue(pic.yDim=actheight);
	locXY->setChecked(pic.locXY);
	qual->setValue(pic.quality);
}


/* *************************************************** */
/* HANDLE SAVE/RESTORE OF WINDOW-COORDINATES           */
/* *************************************************** */
void picWidget::closeEvent(QCloseEvent *event) {
	writeSettings();
	event->accept();
}

void picWidget::readSettings() {
	QSettings settings("Heinz", NAME);
	QPoint pos=settings.value("PicWindowPos", QPoint(200, 200)).toPoint();
	QSize size=settings.value("PicWindowSize", QSize(400, 400)).toSize();

	// Ensure to start with defaults on 1st program-start ever
	everStarted=false;
	everStarted=settings.value("FirstStart").toBool();
	if(everStarted==true) {
		pic.png=settings.value("PNGFormat").toBool();
		pic.quality=settings.value("Quality").toInt();
		resize(size);
		move(pos);	
	}
	else {
		everStarted=true;
		resize(QSize(163, 260));
		move(QPoint(350, 500));
	}
}

void picWidget::writeSettings() {
	QSettings settings("Heinz", NAME);
	settings.setValue("PicWindowPos", pos());
	settings.setValue("PicWindowSize", size());
	settings.setValue("PNGFormat", pic.png);
	settings.setValue("Quality", pic.quality);
	settings.setValue("FirstStart", everStarted);
}
