/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "sysinfo.h"


// Defines
#ifdef WIN32
	#define OS "OS"
	#define CPU "PROCESSOR_ARCHITECTURE"	// x86
#else
	#define OS "OSTYPE"
	#define CPU "CPU"						// i686
	extern char **environ;
#endif


// Globals
extern OGLWidget *oglWid;		// OpenGL-Viewer
extern float MHz;				// CPU clock calculated from main

static QString cpuStr;
static bool oldwin=true;
static bool ogl;
static bool oglhw;


// Check system out
// --------------------------------------------------------------------------
QString systemInfo() {
	QString tmp, numberStr;
	QGLFormat fmt;

	ogl=fmt.hasOpenGL();
	if(!ogl)
		return "\0";

	tmp="OS string = ";
	#ifdef WIN32
		if(getenv(OS)!=NULL) {
			tmp+=getenv(OS)+(QString)("\n");
			cpuStr=(QString)getenv(CPU);
			tmp+="CPU = " + cpuStr + (QString)("\n");
		}
		else {
			tmp+="Windows prior XP\n";
			tmp+="CPU = unknown\n";
		}
	#else
		tmp+=(QString)getenv(OS)+(QString)("\n");
		cpuStr=(QString)getenv(CPU);
		tmp+="CPU = " + cpuStr + (QString)("\n");
		oldwin=false;
	#endif

	// MHz was checked out by main() && aniedi->getclock() at program-start
	tmp+="Clock = ";
	if(MHz!=0.0f)
		tmp+=numberStr.number(MHz, 'f', 2)+" MHz\n";
	else
		tmp+="NAN\n";


	if(getenv("MACHTYPE")!=NULL)
		tmp+="MACHTYPE = "+(QString)getenv("MACHTYPE")+(QString)("\n");
	else
		tmp+="MACHTYPE = NAN\n";

	tmp+="Hardware OpenGL : ";
	if(oglWid->format().directRendering()) {
		tmp+="Yes\n\n";
		oglhw=true;
	}
	else {
		tmp+="No\n\n";
		oglhw=false;
	}

	tmp+="Double buffering : ";
	if(oglWid->format().doubleBuffer()) tmp+="Yes\n"; else tmp+="No\n";

	tmp+="RGBA mode : ";
	if(oglWid->format().rgba()) tmp+="Yes\n"; else tmp+="No\n";

	tmp+="Overlay support : ";
	if(oglWid->format().hasOpenGLOverlays()) tmp+="Yes\n"; else tmp+="No\n";

	tmp+="Multisample support : ";
	if(oglWid->format().sampleBuffers()) tmp+="Yes\n"; else tmp+="No\n";

	tmp+="Depth buffer : ";
	if(oglWid->format().depth()) tmp+="Yes\n"; else tmp+="No\n";
	tmp+="Depth buffer size = ";
	if(oglWid->format().depth())
		tmp+=numberStr.number(oglWid->format().depthBufferSize())+" KB\n";
	else
		tmp+="NAN\n";

	tmp+="Stencil buffer : ";
	if(oglWid->format().stencil()) tmp+="Yes\n"; else tmp+="No\n";
	tmp+="Stencil buffer size = ";
	if(oglWid->format().stencil())
		tmp+=numberStr.number(oglWid->format().stencilBufferSize())+" KB\n";
	else
		tmp+="NAN\n";

	tmp+="Alpha buffer : ";
	if(oglWid->format().alpha()) tmp+="Yes\n"; else tmp+="No\n";
	tmp+="Alpha buffer size = ";
	if(oglWid->format().alpha())
		tmp+=numberStr.number(oglWid->format().alphaBufferSize())+" KB\n";
	else
		tmp+="NAN\n";

	tmp+="Accumulation buffer : ";
	if(oglWid->format().accum()) tmp+="Yes\n"; else tmp+="No\n";
	tmp+="Accumulation buffer size = ";
	if(oglWid->format().accum())
		tmp+=numberStr.number(oglWid->format().accumBufferSize())+" KB\n\n";
	else
		tmp+="NAN\n\n\n";

	return tmp;
}

// Check out CPU
bool cpux86() {
	systemInfo();

	if( cpuStr.startsWith("x86") ||		// Windows XP
		cpuStr.startsWith("i586") ||
		cpuStr.startsWith("i686") )		// Linux
		return true;

	return false;
}

// Check out XP
bool winXP() {
	systemInfo();
	return (!oldwin);
}

// Check for OpenGL
bool ogl_avail() {
	systemInfo();
	return ogl;
}

// Check for HW-OpenGL
bool oglhw_avail() {
	systemInfo();
	return oglhw;
}

