/****************************************************************************
**
** Copyright (C) 1991-2006 Heinz van Saanen
**
** This file is part of the the function viewer Zhu3D.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "timers.h"


double secs;


// OS IS WINDOWS with mm-timer resolutions of maximal 1 msec
// ------------------------------------------------------------
#ifdef WIN32
	#include <windows.h>
	#include <mmsystem.h>

	static DWORD tmp;
	static TIMECAPS resolution;

	void initMMTimer() {
		if(timeGetDevCaps(&resolution, sizeof(TIMECAPS)) == TIMERR_NOERROR) {
			#ifdef DEBUG
			PRINT((int)resolution.wPeriodMin);
			PRINT((int)resolution.wPeriodMax);
			#endif
		}
		#ifdef DEBUG
		else
			PRINT("Timer error");
		#endif
	}

	void TimerStart() {
		if(timeBeginPeriod(1)==TIMERR_NOERROR) {
			tmp=timeGetTime();
			timeEndPeriod(1);
		}
		#ifdef DEBUG
		else
			PRINT("Timer error");
		#endif
	}

	void TimerStop() {
		if(timeBeginPeriod(1)==TIMERR_NOERROR) {
			secs=(timeGetTime()-tmp)/1000.0;
			timeEndPeriod(1);
		}
		#ifdef DEBUG
		else
			PRINT("Timer error");
		#endif
	}

	unsigned long long sysTicks() {
		if(timeBeginPeriod(1)==TIMERR_NOERROR) {
			tmp=timeGetTime();
			timeEndPeriod(1);
			return tmp;
		}
		#ifdef DEBUG
		else
			PRINT("Timer error");
		#endif
		return 0UL;
	}


// ELSE LINUX with default timer resolutions 1 usec
// ------------------------------------------------------------
#else
	#include <sys/time.h>
	#include <unistd.h>

	// Hold time values for useconds
	static struct timeval  tvu1, tvu2;
	static struct timezone tz;
	
	void TimerStart() {
		gettimeofday(&tvu1,&tz);
	}

	void TimerStop() {
		gettimeofday(&tvu2,&tz);
		secs=(tvu2.tv_sec-tvu1.tv_sec)+(tvu2.tv_usec-tvu1.tv_usec)/1000000.0;
	}

	unsigned long long sysTicks() {
		gettimeofday(&tvu1,&tz);
		return (tvu1.tv_sec+tvu1.tv_usec);
	}
#endif


// CPU-cycle-counter
// ---------------------------------------------------------------------------------
#if defined(__i386__) || defined(__x86_64__)
	unsigned long long cpuTicks() {
		unsigned long long tmp;
		__asm__ volatile(".byte 0x0f, 0x31" : "=A" (tmp));
		return (tmp);
	}

#elif defined(__powerpc__)
	unsigned long long cpuTicks(void) {
		unsigned long long result=0;
		unsigned long upper, lower, tmp;
		__asm__ volatile(
			"loop:                  \n"
			"\tmftbu   %0           \n"
			"\tmftb    %1           \n"
			"\tmftbu   %2           \n"
			"\tcmpw    %2,%0        \n"
			"\tbne     loop         \n"
			: "=r"(upper),"=r"(lower),"=r"(tmp)
		);
		result=upper;
		result=result<<32;
		result=result|lower;
		return (result);
	}
#endif
